
#define IN
#define OUT

typedef void *FIND_HANDLE;
typedef void *STRING_BUFFER;
typedef unsigned long SYS_TIME;
typedef unsigned long FILE_ATTRIB;

typedef struct _FILE_ENTRY
{
    //
    // Data structure relationship
    //
    struct _FILE_ENTRY *pSelf;    // The address of this data structure
    struct _FILE_ENTRY *pParent;  // The address of parent directory of this data structure
    struct _FILE_ENTRY *pSibling; // Next file or dir in the same directory

    struct
    {
        size_t nCount;              // How many directories in this dir.
        struct _FILE_ENTRY *pFirst; // The first dir entry.
    } Dirs;

    struct
    {
        size_t nCount;              // How many files in this dir.
        struct _FILE_ENTRY *pFirst; // The first file entry.
    } Files;

    //
    // File information
    //
    struct
    {
        time_t Create;
        time_t LastAccess;
        time_t LastModified;
    } Time;

    FILE_ATTRIB Attrib;
    struct
    {
        size_t nLength;
        char szText[]; // This field MUST be placed at the end of struct.
    } Name;
} FILE_ENTRY;

typedef struct _FILEINFO
{
    FILE_ATTRIB Attrib;
    uint64_t nFileSize;
    uint64_t nPhysicalSize;

    struct
    {
        SYS_TIME Creation;
        SYS_TIME LastAccess;
        SYS_TIME LastWrite;
    } Time;

    struct
    {
        bool bWithNonAscii;
        size_t nLength;
        char szText[255 * 4];
    } Name;
} FILEINFO;

//================================================================================
// Function pointer declaration
//================================================================================
typedef char *(*RTLDIR_GET_CWD)(void);

typedef FIND_HANDLE (*RTLDIR_FIND_FILE_FIRST)(
    IN char *const szPath,
    OUT FILEINFO *const pFileInfo);

typedef bool (*RTLDIR_FIND_FILE_NEXT)(
    IN FIND_HANDLE const hFindHandle,
    OUT FILEINFO *const pFileInfo);

typedef bool (*RTLDIR_FIND_FILE_CLOSE)(
    IN FIND_HANDLE const hFindHandle);

typedef const FILE_ENTRY *(*RTLDIR_READ_DIR)(
    IN OUT char *const szPath,
    IN bool const bRecursive);

typedef bool (*RTLDIR_FULL_PATH)(
    IN FILE_ENTRY *const pFileEntry,
    OUT STRING_BUFFER *const pFullPathname);

typedef FILE_ENTRY *(*RTLDIR_GET_NEXT_SIBLING)(
    IN FILE_ENTRY *const pFileEntry);

typedef bool (*RTLDIR_IS_EXIST)(
    IN char *const szDir);

typedef bool (*RTLDIR_CREATE)(
    IN char *const szDir);

typedef bool (*RTLDIR_DELETE)(
    IN char *const szDir);

//================================================================================
// Module declaration
//================================================================================
#define DIR_PUBLIC_DATA_DECLARATION() \
    bool IsValid

#define DIR_PUBLIC_FUNC_DECLARATION()     \
    RTLDIR_GET_CWD GetCwd;                \
    RTLDIR_IS_EXIST IsExist;              \
    RTLDIR_CREATE Create;                 \
    RTLDIR_DELETE Delete;                 \
    RTLDIR_FIND_FILE_FIRST FindFileFirst; \
    RTLDIR_FIND_FILE_NEXT FindFileNext;   \
    RTLDIR_FIND_FILE_CLOSE FindFileClose; \
    RTLDIR_FULL_PATH FullPath;            \
    RTLDIR_READ_DIR ReadDir;              \
    RTLDIR_GET_NEXT_SIBLING GetNextSibling

/**
 * @module      {Composite} `DIR_MODULE` Directory module
 * @member      {DIR_PUBLIC_FUNC_DECLARATION}
 * @namespace   RtLib.Dir
 * @author      dougpuob <dougpuob@gmail.com>
*/
struct _DIR_MODULE
{
    DIR_PUBLIC_DATA_DECLARATION();
    DIR_PUBLIC_FUNC_DECLARATION();
};